package top.ninwoo.edgecenter.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import top.ninwoo.edgecenter.entity.ClusterConfig;
import top.ninwoo.edgecenter.entity.ContainerDescription;
import top.ninwoo.edgecenter.service.ClusterService;
import top.ninwoo.utils.entity.DockerContainer;
import top.ninwoo.utils.service.DockerService;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author joliu
 * @date 2019-10-18
 * @description 集群服务类
 */
@Service
public class ClusterServiceImpl implements ClusterService {
    private ConcurrentHashMap<Long, Map<String, Set<String>>> clustersInfo = new ConcurrentHashMap<>();
    @Autowired
    DockerService dockerService;

    /**
     * 定时更新容器列表
     */
    /*@Scheduled(fixedRate = 5000)
    @Override
    public void update() {
        dockerService.updateContainers();
    }*/

    @Override
    public List<DockerContainer> getContainerIds() {
        return getContainerIds(false);
    }

    /**
     * 返回容器的id
     * @param isAll
     * @return
     */
    @Override
    public List<DockerContainer> getContainerIds(boolean isAll) {
        if(isAll) {
            return dockerService.getAllContainers();
        } else {
            return dockerService.getRunningContainers();
        }
    }

    /**
     * ＠description 创建容器组
     * @param clusterConfig
     * @return
     */
    @Override
    public long initContainers(ClusterConfig clusterConfig) {
        // 校验ClusterConfig是否符合要求
        if(clusterConfig.getId() == 0L) {
            // 需要保存到数据库中，获取全局唯一的集群id
            saveClusterConfig(clusterConfig);
        }
        if(clustersInfo.containsKey(clusterConfig.getId())) {
            clustersInfo.get(clusterConfig.getId()).clear();
        } else {
            clustersInfo.put(clusterConfig.getId(), new HashMap<String, Set<String>>());
        }
        // 启动容器
        List<ContainerDescription> dockers = clusterConfig.getDockers();
        dockers.forEach(c -> {
            // 创建docker的逻辑
            // TODO: 根据Mode选择不同的模式
            // 根据副本数量创建多少个容器
            if(c.getReplicas() <= 0) {
                // do nothing
            } else {
                DockerContainer containerConfig = c.getDockerContainer();
                String containerName = containerConfig.getName();
                // 校验Container名称是否唯一
                if(clustersInfo.get(clusterConfig.getId()).containsKey(containerName)) {
                    throw new RuntimeException("Container Name must be unique!");
                }
                clustersInfo.get(clusterConfig.getId()).put(containerName, new HashSet<>());
                for (int i = 0; i < c.getReplicas(); i++) {
                    // 处理DockerContainer中的名称
                    if(!"".equals(containerConfig.getName())) {
                        containerConfig.setName(containerName+ "_" + i);
                    }
                    DockerContainer dockerContainer = dockerService.runDocker(containerConfig);
                    clustersInfo.get(clusterConfig.getId()).get(containerName).add(dockerContainer.getId());
                }
            }
        });
        clusterConfig.setCreateTime(new Date());
        return clusterConfig.getId();
    }

    /**
     * @description 保存配置文件到数据库
     * @param clusterConfig
     */
    public ClusterConfig saveClusterConfig(ClusterConfig clusterConfig) {
        // TODO: 保存数据到数据库
        clusterConfig.setId(11111L);
        return clusterConfig;
    }

    /**
     * @description 获取集群Id集合
     * @return
     */
    @Override
    public Set<Long> getClusterIds() {
        return clustersInfo.keySet();
    }

    /**
     * @description 获取集群中的指定容器的容器组集合
     * @param clusterId
     * @param containerName
     * @return
     */
    @Override
    public Set<String> getContainerIdsByClusterId(long clusterId, String containerName) {
        return clustersInfo.get(clusterId).get(containerName);
    }

    /**
     * @description 删除指定集群的全部容器
     * @param clusterId
     */
    @Override
    public void removeContainersByClusterId(long clusterId) {
        if(!clustersInfo.containsKey(clusterId)) {
            return;
        }
        clustersInfo.get(clusterId).keySet().forEach(
                c -> removeContainersByClusterIdAndContainerName(clusterId, c)
        );
        clustersInfo.remove(clusterId);
    }

    /**
     * @description 删除指定容器
     * @param clusterId
     * @param containerName
     */
    @Override
    public void removeContainersByClusterIdAndContainerName(long clusterId, String containerName) {
        if(!clustersInfo.containsKey(clusterId)) {
            return;
        }
        clustersInfo.get(clusterId).get(containerName).forEach(
                c -> dockerService.deleteDockerById(c)
        );
        clustersInfo.get(clusterId).remove(containerName);
    }
}