package top.ninwoo.utils.util.impl;

import org.springframework.beans.factory.annotation.Autowired;
import top.ninwoo.utils.entity.BridgePort;
import top.ninwoo.utils.entity.Ovs;
import top.ninwoo.utils.entity.OvsBridge;
import top.ninwoo.utils.util.LinuxCtlUtils;
import top.ninwoo.utils.util.OvsUtils;
import top.ninwoo.utils.util.Utils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author joliu
 * @Description
 * @Date　Create in 下午9:29 2019/10/20
 */
@Utils
public class OvsUtilsImpl implements OvsUtils {
    @Autowired
    LinuxCtlUtils linuxCtlUtils;

    @Override
    public boolean isInstall() {
        // 可能会遇到权限问题
        String s = linuxCtlUtils.runCmd("ovs-vsctl --version");
        if(s.contains("Error")) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public Ovs showDetails() {
        String res = linuxCtlUtils.runCmd("echo 'Vudo3423' | sudo -S ovs-vsctl show");

        Ovs ovs = parseOvsString(res);
        return ovs;
    }

    /**
     * 将返回的字符串转换为Ovs数据结构
     * @param ovsString
     * @return
     */
    public Ovs parseOvsString(String ovsString) {
        if(ovsString == null || "".equals(ovsString)) {
            throw new RuntimeException("ovs String cannot be null or empty!");
        }
        String[] results = ovsString.split("\n");
        if(results.length < 2) {
            throw new RuntimeException("Illegal ovs Result!");
        }
        Ovs ovs = new Ovs();
        String ovsId = results[0].substring(0,results[0].length()-1);
        ovs.setId(ovsId);
        String ovsVersion = getValueInString(results[results.length - 1]);
        ovs.setOvsVersion(ovsVersion);

        int i = 0;
        OvsBridge bridge = null;
        BridgePort port = null;
        List<BridgePort> ports = new ArrayList<>();
        List<OvsBridge> bridges = new ArrayList<>();
        while(i < results.length) {
            // 创建Bridge
            if(results[i].contains("Bridge")) {
                port = null;
                ports.clear();
                if(bridge != null) {
                    // 放入到数组中
                    bridges.add(bridge);
                }
                String bridgeId = getValueInString(results[i]);
                bridge = new OvsBridge();
                bridge.setBridgeId(bridgeId);
            }
            else if(results[i].contains("Port")) {
                if(port != null) {
                    ports.add(port);
                }
                String portId = getValueInString(results[i]);
                port = new BridgePort();
                port.setPortId(portId.replace("\"", ""));
            }
            else if(results[i].contains("Interface")) {
                if(port == null) {
                    throw new RuntimeException("Cannot find Port");
                }
                String interfaceId = getValueInString(results[i]);
                port.setInterfaceId(interfaceId);
            }
            else if(results[i].contains("Error")) {
                if(port == null) {
                    throw new RuntimeException("Cannot find Port");
                }
                String error = getValueInString(results[i]);
                port.setError(error);
            } else if(results[i].contains("ovs_version")) {
                if(port != null) {
                    ports.add(port);
                    bridge.setPorts(ports);
                    bridges.add(bridge);
                }
                ovs.setBridges(bridges);
            }
            i++;
        }

        return ovs;
    }

    public String getValueInString(String input) {
        String value = input.trim().split(" ")[1];
        value = value.replaceAll("\"", "");
        return value;
    }

    @Override
    public void addBridge(String name) {
        String cmd = "echo 'Vudo3423' | sudo -S ovs-vsctl add-br " + name;
        String res = linuxCtlUtils.runCmd(cmd);
        System.out.println(res);
        if(res.contains("Error")) {
            throw new RuntimeException("linux bridge has existed!");
        }
    }

    @Override
    public void setBridgeProtocol(String bridgeName, String protocol) {
        String cmd = "echo 'Vudo3423' | sudo -S ovs-vsctl set bridge " + bridgeName + " protocols=" + protocol;
        String res = linuxCtlUtils.runCmd(cmd);
        if(res.contains("Error")) {
            throw new RuntimeException(res);
        }
    }

    @Override
    public void setController(String bridgeName, String host, int port) {
        String cmd = "echo 'Vudo3423' | sudo -S ovs-vsctl set-controller " + bridgeName + " tcp:" + host + ":" + port;
        String res = linuxCtlUtils.runCmd(cmd);
        if(res.contains("Error")) {
            throw new RuntimeException(res);
        }
    }

    @Override
    public void delBridge(String name) {
        String cmd = "echo 'Vudo3423' | sudo -S ovs-vsctl del-br " + name;
        String res = linuxCtlUtils.runCmd(cmd);
        if(res.contains("Error")) {
            throw new RuntimeException("bridge not found!");
        }
    }
}
